// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import C25519Key from './c25519Key';
import CompositeCipher from './compositeCipher';

export default class CompositeKey {
    /** @param {C25519Key} key */
    constructor(key = null) {
        this.key = key ? key : C25519Key.generate();
    }
    
    /** @param {Uint8Array|string} data */
    encrypt(data) {
        return CompositeCipher.encrypt(data, this.key);
    }
    
    /** @param {Uint8Array|string|CompositeCipher} data */
    decrypt(data) {
        const cipher = data instanceof CompositeCipher ? data : CompositeCipher.from(data);
        return cipher.decrypt(this.key);
    }

    public() {
        return new CompositeKey(this.key.public());
    }

    toString() {
        return this.key.toString();
    }

    toArray() {
        return this.key.toArray();
    }

    /** @param {import('./bnInput').BNInput} x
     * @param {boolean} noPublic */
    static private(x, noPublic = false) {
        return new CompositeKey(C25519Key.private(x, noPublic));
    }

    static generate() {
        return new CompositeKey();
    }

    /** @param {string | Uint8Array} data */
    static from(data) {
        return new CompositeKey(C25519Key.from(data));
    }
}
